import { And } from './and';
import { IsEqual } from './is-equal';
import { Load } from './load';
import { Match } from './match';
import { MulAdd } from './mul-add';
import { Or } from './or';
import { Span } from './span';
import { Store } from './store';
import { Test } from './test';
import { Update } from './update';
import { Value } from './value';

export default {
  And, IsEqual, Load, Match, MulAdd, Or, Span, Store, Test, Update, Value,
};
